//【登録場所】リンク、URLExec、選択テキスト(ユーザーポップアップのみ対応)
//【ラベル】ツイートのポップアップ
//【内容】ツイート、ツイッターユーザーのポップアップ
//     ツイートかユーザーかはURLから自動的に判別
//     選択テキストから起動された場合は、それをユーザーIDとして処理する
//【コマンド1】${SCRIPT:FrS} popupTwitterInfo.js
//【コマンド2】${SCRIPT:FrS} popupTwitterInfo.js テンプレートのファイル名       (popupTweetフォルダ内の任意のテンプレートファイルを指定)
//【URLExec】https?://(?:\w+\.)?twitter\.com/(?:#!/)?(\w+)[#/]?$	http://twitter.com/$1	${V2CSCRIPT:FrS} popupTwitterInfo.js　（ユーザー）
//           https?://(?:(?:\w+\.)?twitter\.com|twtr.jp/user)/(?:#!/)?([^/])+/status(?:es)?/(\d+)(?!/photo)	http://twitter.com/$1/status/$2[TAB}${V2CSCRIPT:FrS}　（ツイート）

//設定(onがtrue、offがfalse)-----------------------
//ポップアップの最大幅
var maxPopupWidth = 400;

//マウス移動でポップアップを閉じる
v2c.context.setDefaultCloseOnMouseExit(true);

//終了後に残りアクセス回数をステータスバーに表示
var showRemainingHits = false;

//一度読み込んだテンプレートとtwitterデータはキャッシュし、同じデータを読み込む場合はこれを表示する。
//キャッシュはV2C再起動でクリアされる。使用にはT20110522以降が必要。
var cacheMode = false;

//ポップアップ上の@userクリックをスクリプトによるポップアップするかどうか
var userPopup = false;
//-----------------------------------------------

//デフォルトテンプレートファイル名
var TEMPLATE_TWEET = "TemplateStatus.txt";
var TEMPLATE_USER = "TemplateUser.txt";

//キャッシュ管理
var cacheData = {
  data: null,
  load: function() {
    if (cacheMode) {
      this.data = v2c.getScriptObject();
    }
    if (!this.data) {
      this.data = {};
    }
  },
  set: function(key, value) {
    this.load();
    this.data[key] = value;
    if (cacheMode) {
      v2c.setScriptObject(this.data);
    }
  }, 
  get: function(key) {
    this.load();
    return this.data[key];
  }
}

var userRx =/https?:\/\/(?:\w+\.)?twitter\.com\/(?:#!\/)?(\w+)/i;
var statusRx =/https?:\/\/(?:\w+\.)?twitter\.com\/(?:#!\/)?[^\/]+\/status(?:es)?\/(\d+)/i;

var u = v2c.context.link;
if (!u) {//選択テキストから起動された場合は、ユーザーIDとして処理する
  var selectedText = v2c.getSelectedText();
  if (selectedText && selectedText.match(/@?([0-9A-Za-z_]{1,15})/)) {
    u = "http://twitter.com/" + RegExp.$1;
  }
}
if (u) {
  popupTwitterInfo(u.toString(), false);
}
else {
  v2c.alert('URL取得失敗');
}

function popupTwitterInfo(url,isRefresh) {

  var html,key,getHtml;
  var tm = v2c.context.args[0];

  //URLが正しいかどうかの確認
  if (url.match(statusRx)) {
    templateFilename = tm ? tm : TEMPLATE_TWEET;
    getHtml = getTwitterStatusHTML;
  }
  else if (url.match(userRx)) {
    templateFilename = tm ? tm : TEMPLATE_USER;
    getHtml = getTwitterUserHTML;
  }
  else {
    v2c.alert("非対応のURLです\n" + url);
    return;
  }
  
  if (getHtml) {
    key = RegExp.$1 + '/' + templateFilename;
    if (!isRefresh) {
      //同じURLのポップアップを開いていたら終了
      if (v2c.context.getPopupOfID(key)) {
        return;
      }
      html = cacheData.get(key);
    }
    if (!html){
      html = getHtml(RegExp.$1, templateFilename);
    }
  }
  if (html) {
    //ポップアップの設定
    html = html.replace('%url%',url);//更新ボタン用
    v2c.context.setPopupHTML(html);
    v2c.context.setMaxPopupWidth(maxPopupWidth);
    v2c.context.setPopupID(key);
    v2c.context.setRedirectURL(true);
    v2c.context.setCloseOnLinkClick(!userPopup);
    v2c.context.setTrapFormSubmission(true);
    cacheData.set(key,html);
  }
  
  if (showRemainingHits) {
    var limit = getJson("http://api.twitter.com/1/account/rate_limit_status.json");
    if (limit) {
      v2c.context.setStatusBarText("残り回数：" + limit.remaining_hits +
                                    " 次のリセット時間：" +
                                    getDateText(limit.reset_time));
    }
  }
  return;
}

function formSubmitted(url, sm, sd) {
  v2c.context.closeOriginalPanel();
  popupTwitterInfo(url.toString(),true);
}

function redirectURL(url) {
  if (userPopup) {
    url = url + '';
    if (!url.match(statusRx) && url.match(userRx) && url.indexOf('#!') == -1) {
      popupTwitterInfo(url, false);
      return null;
    }
  }
  return url;
}
function getTwitterStatusHTML(sid,template) {

  var url = "http://api.twitter.com/1/statuses/show/" + sid + ".json";
  var json = getJson(url);
  if (!json) {
    return null;
  }
  
  //テンプレートを読み込み
  var templateText = readTemplate(template);
  if (!templateText) {
    v2c.alert("ファイルがない？ " + template);
    return null;
  }
  
  var html = getTwitterUserFromJson(json.user, templateText);
  html = getTwitterStatusFromJson(json, html);
  
  return html;
}

function getTwitterUserHTML(user,template) {
  var url = "http://api.twitter.com/1/users/show/" + user + ".json";
  var json = getJson(url);
  if (!json) {
    return null;
  }
  
  //テンプレートを読み込み
  var templateText = readTemplate(template);
  if (!templateText) {
    v2c.alert("ファイルがない？ " + template);
    return null;
  }
  
  var html = getTwitterUserFromJson(json, templateText);
  //最新ツイート取得
  //if (json.statuses_count > 0) {
  html = getTwitterStatusFromJson(json.status, html);
  //}
  
  return html;
}


function getTwitterStatusFromJson(statusJson, templateText) {
  var text = '';
  var date = '';
  var client = '';
  var retweetCount = '0';
  var statusID = '0';
  
  if (statusJson) {
    //本文の取得
    if (statusJson.text) {
      text = addLinkTag(statusJson.text + '');
    }
    
    //投稿日時の取得
    if (statusJson.created_at) {
      date = getDateText(statusJson.created_at);
    }
    
    //リツイート数の取得
    if (statusJson.retweet_count) {
      retweetCount = statusJson.retweet_count + '';
    }
    
    //クライアント
    if (statusJson.source) {
      client = statusJson.source;
    }
    
    //statusID取得
    if (statusJson.id) {
      statusID = statusJson.id + '';
    }
  }
  else {
    text = "取得できず"
  }
  
  templateText = templateText.replace('%date%', date);
  templateText = templateText.replace('%sid%', statusID);
  templateText = templateText.replace('%via%', client);
  templateText = templateText.replace('%retweet%', retweetCount);
  templateText = templateText.replace('%text%', text);
  
  return templateText;
}

function getTwitterUserFromJson(userJson, templateText) {
  var followersCount = '0';
  var friendsCount = '0';
  var statusesCount = '0';
  var verified = '';
  var createdAt = '';
  var icon = '';
  var icolink = '';
  var name = '';
  var screenName = '';
  var homeurl = '';
  var description = '';
  
  if (userJson) {
    //フォロワー数取得
    if (userJson.followers_count) {
      followersCount = userJson.followers_count + '';
    }
    
    //フォロー数取得
    if (userJson.friends_count) {
      friendsCount = userJson.friends_count + '';
    }
    
    //ツイート数
    if (userJson.statuses_count) {
      statusesCount = userJson.statuses_count + '';
    }
    
    //urlの取得
    if (userJson.url) {
      homeurl = userJson.url;
    }
    
    //認証済み
    if (userJson.verified) {
      verified = '認証済み';
    }
    
    //アカウント名の取得
    if (userJson.screen_name) {
      screenName = userJson.screen_name;
    }
    
    //アイコンURL取得
    if (userJson.profile_image_url) {
      icon = userJson.profile_image_url;
      icolink = "http://twitter.com/#!/" + screenName;
    }
    
    //表示名取得
    if (userJson.name) {
      name = userJson.name;
    }
    
    //紹介文の取得
    if (userJson.description) {
      description = addLinkTag(userJson.description + '');
    }
    
    //作成日の取得
    if (userJson.created_at) {
      createdAt = getDateText(userJson.created_at,"yyyy/MM/dd");
    }
  }
  //パラメータの置換
  templateText = templateText.replace('%aname%', screenName);
  templateText = templateText.replace('%uname%', name);
  templateText = templateText.replace('%icon%', icon);
  templateText = templateText.replace('%icolink%', icolink);
  templateText = templateText.replace('%verified%', verified);
  templateText = templateText.replace('%followers_count%', followersCount);
  templateText = templateText.replace('%statuses_count%', statusesCount);
  templateText = templateText.replace('%friends_count%', friendsCount);
  templateText = templateText.replace('%created_at%', createdAt);
  templateText = templateText.replace('%homeurl%', homeurl);
  templateText = templateText.replace('%description%', description);
  return templateText;
}

function addLinkTag(htmlText) {
  var http = "(?:(?:ftp|https?)://[-_.!~*'()a-zA-Z0-9;/?:@&=+$,%#]+)";
  var user = '(?:[@][0-9A-Za-z_]{1,15})';
  var hashtag = '(?:(^|[^a-zA-Z0-9&?]+)#(\\w*[a-zA-Z_]\\w*))';
  
  var r = new RegExp([http, user, hashtag].join('|'), 'g');
  return htmlText.replace(r, function(m0) {
    if (m0.match(/^(?:ftp|http)/)) {
      return '<a href="' + m0 + '">' + m0 + '</a>';
    }
    else if (m0.match(/^@/)) {
      return '@<a href="http://twitter.com/' +
                m0.substr(1) + '">' + m0.substr(1) + '</a>';
    }
    else if (m0.match('^' + hashtag)) {
      return RegExp.$1 + '<a href="http://twitter.com/search?q=' +
               encodeURIComponent('#' + RegExp.$2) + '">#' + RegExp.$2 + '</a>';
    }
    else {
      return m0;
    }
  })
}

function getDateText(d, pattern) {
  var dd = new Date(d);
  if (!pattern) {
    pattern = "yyyy/MM/dd HH:mm:ss (E)";
  }
  var sdf = new java.text.SimpleDateFormat(pattern);
  return sdf.format(dd);
}

function getJson(url) {
  v2c.setStatus('popupTwitterInfo通信中...');
  var hr = v2c.createHttpRequest(url);
  var sr = hr.getContentsAsString();
  if (!sr) {
    v2c.context.setStatusBarText('PopupTwitterInfo ページの取得に失敗しました。: ' + hr.responseCode + ' ' + hr.responseMessage + ' ' + url);
    return null;
  }
  
  //データ取得
  return eval("(" + sr + ")");
}

function readTemplate(fileName) {
  var template = cacheData.get(fileName);
  if (!template) {
    template = v2c.readFile(combinePath(v2c.saveDir, 'script', 'popupTweet', fileName));
    cacheData.set(fileName, template);
  }
  return template;
}

function combinePath() {
  if (arguments.length == 0) {
    return "";
  }
  
  var combined = new java.io.File(arguments[0]);
  for (var i = 1; i < arguments.length; i++) {
    combined = new java.io.File(combined, arguments[i]);
  }
  return combined.getPath();
}
